@@ -4,6 +4,7 @@ import datetime
 import traceback
 from django.db import transaction
 from django_future.models import ScheduledJob
+from django_future.utils import parse_timedelta
 
 
 __all__ = ['schedule_job', 'job_as_parameter']
@@ -26,13 +27,13 @@ def schedule_job(date, callable_name, content_object=None, expires='7d',
     """
     assert callable_name and isinstance(callable_name, str), callable_name
     if isinstance(date, basestring):
-        date = _parse_timedelta(date)
+        date = parse_timedelta(date)
     if isinstance(date, datetime.timedelta):
         date = datetime.datetime.now() + date
     job = ScheduledJob(callable_name=callable_name, time_slot_start=date)
     if expires:
         if isinstance(expires, basestring):
-            expires = _parse_timedelta(expires)
+            expires = parse_timedelta(expires)
         if isinstance(expires, datetime.timedelta):
             expires = date + expires
         job.time_slot_end = expires
@@ -44,18 +45,6 @@ def schedule_job(date, callable_name, content_object=None, expires='7d',
     return job
 
 
-_TIMEDELTA_SUFFIXES = {'m': 'minutes',
-                       'h': 'hours',
-                       'd': 'days',
-                       'w': 'weeks'}
-
-def _parse_timedelta(s):
-    n, suffix = int(s[:-1]), s[-1]
-    key = _TIMEDELTA_SUFFIXES[suffix]
-    kwargs = {key: n}
-    return datetime.timedelta(**kwargs)
-
-
 def job_as_parameter(f):
     f.job_as_parameter = True
     return f
@@ -68,14 +57,18 @@ def run_jobs(delete_completed=False, ignore_errors=False, now=None):
     You may specify a date to be treated as the current time.
     """
     if ScheduledJob.objects.filter(status='running'):
-        # TODO: this kind of test is unreliable: the other job processor
-        # could be inbetween jobs at the time of this check.
         raise ValueError('jobs in progress found; aborting')
     if now is None:
         now = datetime.datetime.now()
+    # Expire jobs.
+    expired_jobs = ScheduledJob.objects.filter(status='scheduled',
+                                               time_slot_end__lt=now)
+    for job in expired_jobs:
+        job.status = 'expired'
+        job.save()
+    # Get scheduled jobs.
     jobs = ScheduledJob.objects.filter(status='scheduled',
-                                       time_slot_start__lte=now,
-                                       time_slot_end__gt=now)
+                                       time_slot_start__lte=now)
     for job in jobs:
         job.status = 'running'
         job.execution_start = datetime.datetime.now()
@@ -87,11 +80,10 @@ def run_jobs(delete_completed=False, ignore_errors=False, now=None):
             exc_text = traceback.format_exc()
             transaction.rollback()
             job.error = exc_text
-            # TODO: complain somehow
             job.status = 'failed'
             job.save()
-            transaction.commit()
             if not ignore_errors:
+                transaction.commit()
                 raise
         else:
             transaction.commit()
@@ -100,4 +92,8 @@ def run_jobs(delete_completed=False, ignore_errors=False, now=None):
             else:
                 job.status = 'complete'
                 job.save()
-            transaction.commit()
+        # Do not commit the transaction here on purpose, so that
+        # the next job is marked as active and the current one is marked as
+        # complete in the same transaction.
+
+    transaction.commit()
